@@ -113,8 +113,9 @@ def set_security_level(action, cmdpairs):
 			sec_level = "all"
 			rdir = getpath(cp2)
 	elif action in ["test-server", "list-increments", 'list-increment-sizes',
-					 "list-at-time", "list-changed-since",
-					 "calculate-average", "remove-older-than", "compare"]:
+					"list-at-time", "list-changed-since",
+					"calculate-average", "remove-older-than", "compare",
+					"compare-hash", "compare-full", "verify"]:
 		sec_level = "minimal"
 		rdir = tempfile.gettempdir()
 	else: assert 0, "Unknown action %s" % action
@@ -151,7 +152,15 @@ def set_allowed_requests(sec_level):
 				  "restore.ListAtTime",
 				  "backup.SourceStruct.get_source_select",
 				  "backup.SourceStruct.set_source_select",
-				  "backup.SourceStruct.get_diffs"])
+				  "backup.SourceStruct.get_diffs",
+				  "compare.RepoSide.init_and_get_iter",
+				  "compare.RepoSide.close_rf_cache",
+				  "compare.RepoSide.attach_files",
+				  "compare.DataSide.get_source_select",
+				  "compare.DataSide.compare_fast",
+				  "compare.DataSide.compare_hash",
+				  "compare.DataSide.compare_full",
+				  "compare.Verify"])
 	if sec_level == "update-only" or sec_level == "all":
 		l.extend(["log.Log.open_logfile_local", "log.Log.close_logfile_local",
 				  "log.ErrorLog.open", "log.ErrorLog.isopen",
@@ -197,12 +206,12 @@ def vet_request(request, arglist):
 	"""Examine request for security violations"""
 	#if Globals.server: sys.stderr.write(str(request) + "\n")
 	security_level = Globals.security_level
+	if security_level == "override": return
 	if Globals.restrict_path:
 		for arg in arglist:
 			if isinstance(arg, rpath.RPath): vet_rpath(arg)
 		if request.function_string in file_requests:
 			vet_filename(request, arglist)
-	if security_level == "override": return
 	if request.function_string in allowed_requests: return
 	if request.function_string in ("Globals.set", "Globals.set_local"):
 		if arglist[0] not in disallowed_server_globals: return
