@@ -19,7 +19,7 @@
 
 """Manage logging, displaying and recording messages with required verbosity"""
 
-import time, sys, traceback, types
+import time, sys, traceback, types, rpath
 import Globals, static, re
 
 
@@ -203,24 +203,17 @@ class ErrorLog:
 
 	"""
 	_log_fileobj = None
-	_log_inc_rp = None
 	def open(cls, time_string, compress = 1):
 		"""Open the error log, prepare for writing"""
 		if not Globals.isbackup_writer:
 			return Globals.backup_writer.log.ErrorLog.open(time_string,
 														   compress)
-		assert not cls._log_fileobj and not cls._log_inc_rp, "log already open"
+		assert not cls._log_fileobj, "log already open"
 		assert Globals.isbackup_writer
-		if compress: typestr = 'data.gz'
-		else: typestr = 'data'
-		cls._log_inc_rp = Globals.rbdir.append("error_log.%s.%s" %
-											   (time_string, typestr))
-		assert not cls._log_inc_rp.lstat(), ("""Error file %s already exists.
 
-This is probably caused by your attempting to run two backups simultaneously
-or within one second of each other.  Wait a second and try again.""" %
-											 (cls._log_inc_rp.path,))
-		cls._log_fileobj = cls._log_inc_rp.open("wb", compress = compress)
+		base_rp = Globals.rbdir.append("error_log.%s.data" % (time_string,))
+		if compress: cls._log_fileobj = rpath.MaybeGzip(base_rp)
+		else: cls._log_fileobj = cls._log_inc_rp.open("wb", compress = 0)
 
 	def isopen(cls):
 		"""True if the error log file is currently open"""
@@ -267,7 +260,7 @@ or within one second of each other.  Wait a second and try again.""" %
 		if not Globals.isbackup_writer:
 			return Globals.backup_writer.log.ErrorLog.close()
 		assert not cls._log_fileobj.close()
-		cls._log_fileobj = cls._log_inc_rp = None
+		cls._log_fileobj = None
 
 static.MakeClass(ErrorLog)
 
